/*
 *   File:  NewAccountApplet.java
 *
 *   The main applet code that creates a new hushmail account.
 *
 *   version 1.04 v1a
 *   Copyright 1998, 1999 by Hush Communications Corporation, BWI
 */





/*
 *  Applet parameters:
 *
 *  username = < the users HushMail account >
 *  userinfo = < a list of items of data on the user seperated by & >
 *  diskspace = < the diskspace to be allocated to the user in bytes >
 *  loginpage = < URL of HushMail login page >
 *  sessionID = < a 64 bit hex number represented as a string >
 *  sessionKey = < a 128 bit hex number represented as a string >
 *  R0 = < red value for applet color >
 *  G0 = < green value for applet color >
 *  B0 = < blue value for applet color >
 *  R1 = < red value for applet color >
 *  G1 = < green value for applet color >
 *  B1 = < blue value for applet color >
 */

package hush.gui;

import java.awt.*;
import java.applet.*;
import java.net.*;
import java.io.*;
import java.util.*;
import hush.security.HushSHA1;
import hush.security.BlowfishCipher;
import hush.security.ElGamalKeys;
import hush.util.Conversions;
import hush.util.ByteQueue;


final public class NewAccountApplet extends Applet
{
   private static final int KEYSTRENGTH = 1024;
   private boolean keysGenerated = false;
   private String encPrivateKey;
   private String publicKey;

   /**
    *   Variables concerning obtaining random numbers from mouse movement.
    */

   //  Coordinates of the top left corner of the main 256X256 square
   int mainx;
   int mainy;
   int mainwidth = 256;
   int mainheight = 256;

   //  Previous x and y coordinates
   int px;  
   int py;

   //  Arrays of x and y defining several squares
   int numberOfSquares = 128;
   int[] sx = new int[numberOfSquares];
   int[] sy = new int[numberOfSquares];
   int squares[][] = new int[mainwidth][mainheight];
   int sw = 12;
   int sh = 12;

   //  True until random squares have been generated
   private boolean initializing;


   /*  To track which squares have been hit
    *  numberOfsqares (128) indicates that square has not yet been hit (initialized state)
    */
   int square0=numberOfSquares;   //current square
   int square1=numberOfSquares;   //previous squares
   int square2=numberOfSquares;  
  
   //  Every ten zone changes get system time.  Track with this variable.
   int squareCounter = 0;

   // Indicates whether mouse movements still need to be used to queue bits. 
   private boolean getMouse = true;

   private ByteQueue bytes;

   private int bitsNeeded = KEYSTRENGTH;

   private int nextGraphNotchAt = bitsNeeded/32;

   /**
    *  Other variables
    */

   private String serverAddress;
   private int serverPort = -1;
   private String hushHome = "";

   private Random rand;
   long randSeed = 0;
   private Graphics g;
   private Color color;

   private TextField usernameField;

   private String username;
   private String userinfo;
   private String diskspace;
 
   private TextField passphraseField;
   private TextField passphraseConfirmField;

   private Label aLabel;
   private Label bLabel;
   private Button loginButton;
   private Button submitPassphraseButton;
   private Button confirmPassphraseButton;
   private Button submitUsernameButton;

   //  The pseudo stream cipher for communication between applet and server
   private BlowfishCipher blowfishPipe;

   //  The sessionID.  Used by server to locate sessionKey.
   private byte[] sessionIDBytes;

   //  An output stream to the socket
   private BufferedOutputStream out;

   //  An input stream from the socket.
   private BufferedInputStream eIn;

   /* An input stream from a decrypted String which is used as a buffer
    * to store data from eIn after it has been decrypted with blowfishPipe
    */
   private DataInputStream in;

  private Color backgroundColor;  

   public void init()
   {
      username = getParameter("username");
      userinfo = getParameter("userinfo").replace('&','\n')+"\n";

      diskspace= getParameter("diskspace");
      hushHome = getParameter("loginpage");

      color = new Color(Integer.parseInt(getParameter("R1")),
			Integer.parseInt(getParameter("G1")),
			Integer.parseInt(getParameter("B1")));

      serverAddress = getCodeBase().getHost();
      getPorts();

      bytes = new ByteQueue();

      //  Convert the sessionID from a hex string to a byte array
      sessionIDBytes = Conversions.hexStringToBytes(getParameter("sessionID"));

      /* Convert the sessionKey from a hex string to a byte array and use
       * it to instantiate the CBC Blowfish cipher which will encrypt
       * communication between applet and server.
       */
      byte[] sessionKeyBytes = Conversions.hexStringToBytes(getParameter("sessionKey"));
      blowfishPipe = new BlowfishCipher();
      blowfishPipe.setKey(sessionKeyBytes);

      setFont(new Font("Helvetica",Font.PLAIN,12));

      // <<By Zhou Wu 
      backgroundColor = new Color(Integer.parseInt(getParameter("R0")),
			      Integer.parseInt(getParameter("G0")),
			      Integer.parseInt(getParameter("B0")));
      setBackground(backgroundColor);
      //>>
      setForeground(Color.white);
      setLayout(new GridBagLayout());
      aLabel = new Label("You must now generate some random numbers to create your keys.");
      add(this,aLabel,0,0,1,1,0,0,0,0,N,NO,0,0,0,0);
      bLabel = new Label("Move the mouse in the box until the graph fills");
      add(this,bLabel,0,1,1,1,0,0,0,1,N,NO,0,0,0,0);
      add(bLabel); 

      g = this.getGraphics();   
      initializing = true;
   }
   
    private Socket socket;  
    private int portArray[];
	private void getPorts()
	{

	    String ports = getParameter("ports");
	    if (ports != null)
	    {

	        StringTokenizer st = new StringTokenizer(ports,", ;");
	         portArray = new int[ st.countTokens()];
	        //int i=0;
	        for ( int i=0; i< portArray.length; i++)
	        {
	            portArray[i] = Integer.parseInt( st.nextToken());
            
	        }
	       

	    }  
    
	}

   public void paint(Graphics g)
   {
      paintComponents(g);

      /* Draw a rectangle for the user to move the mouse in, and a bar graph
       * to track the random bits enqueued.
       */
      if (getMouse)
      {

         mainx=bounds().x+(bounds().width-mainwidth)/2;
         mainy=bounds().y+aLabel.bounds().height+bLabel.bounds().height+10;
     
         Rectangle r = new Rectangle(mainx,mainy,mainwidth,mainheight);
         g.setColor(color);
         g.fillRect(r.x,r.y,r.width,r.height);

         //  Set up graph based on the length of the desired bit array
         for (int x=mainx; x<mainx+mainwidth; x=x+8)
         {
            Rectangle br = new Rectangle(x+2, mainy+mainheight+10, 4, 20);
            g.fillRect(br.x, br.y, br.width, br.height);
         }
      }

      g.setColor(Color.white);
   }


   public boolean mouseMove(Event e, int x, int y)
   {
      if (x<mainx || y<mainy || x>=mainx+mainwidth || y>=mainy+mainheight)
         return true;
      else if (initializing)
      {
         //  Seed the random number generator
         randSeed = randSeed ^ ((long)x<<40)+((long)y<<32)+new Date().getTime();
         rand = new Random(randSeed);    

         //  Randomly generate a number of squares in the main square
         boolean goodSquare = true;

         for (int n=0; n<numberOfSquares; n++)
            for(; ;)
            {
               goodSquare = true;
               sx[n] = Math.abs(rand.nextInt())%(mainwidth-sw);
               sy[n] = Math.abs(rand.nextInt())%(mainheight-sh);

               for (int nn=0;nn<n;nn++)
                  if (!(sx[n]<sx[nn]-sw||sy[n]<sy[nn]-sh||sx[n]>sx[nn]+sw||sy[n]>sy[nn]+sh))
                  {
                     goodSquare = false;
                     break;
                  }
               if (goodSquare) 
                  break;
            }
      
         //  Clear the squares
         squares = new int[mainheight][mainwidth];
         for (int n=0; n<numberOfSquares; n++)
            for (int xx=sx[n]; xx<sx[n]+sw; xx++)
               for (int yy=sy[n]; yy<sy[n]+sh; yy++) 
                  squares[xx][yy] = n+1;

         initializing = false;

/*       //  For debugging, show squares.
         Rectangle r = new Rectangle(mainx,mainy,mainwidth,mainheight);
         g.setColor(color);
         g.fillRect(r.x,r.y,r.width,r.height);
         g.setColor(Color.white);
         for (int n=0; n<numberOfSquares; n++)
            g.fillRect(mainx+sx[n],mainy+sy[n],sw,sh);
*/
      }


      else if (getMouse)
      {
         boolean inSquare = false;  
         int n = squares[x-mainx][y-mainy]-1;

         if (n!=-1)
         {	
	    //System.out.println("Hit square "+n);

            square2 = square1;
            square1 = square0;
            square0 = n;

            /*  Exit if square is one of past two hit, or this is one of first 
             *  three squares hit (initialized state - no history) 
             */
            if (square0==square1 || square0==square2 || square1==numberOfSquares ||
                square2==numberOfSquares)  return true;
            inSquare = true;
         }

         if (!inSquare) return true;
 
         //  Get first five bits (5 LSB's of square #)
         bytes.enqueueBits(square0,5);

         /* Split square in half vertically (and then horizontally)
          * and take one bit based on side mouse is on.
          */ 
         bytes.enqueueBits( (x-sx[square0] < (sw/2) ) ? 0:1 , 1);
         bytes.enqueueBits( (y-sy[square0] < (sh/2) ) ? 0:1 , 1);

         /*  Get eighth bit
          *  (left or right of square) xor (top or bottom of square) xor (MSB of square #)
          */
         if ( ((x-sx[square0] < (sw/2)) ^ (y-sy[square0] < (sh/2) ) ^ 
              (square0 < (numberOfSquares / 2))))  bytes.enqueueBits(0,1);
         else  bytes.enqueueBits(1,1);

         if (squareCounter++ == 8) 
         {
            initializing = true;
            squareCounter = 0;
         }

         if (bytes.bitsEnqueued()-1 >= nextGraphNotchAt)
         {
            //  Fill another notch in the graph
            Rectangle br = new Rectangle(
               mainx + ((bytes.bitsEnqueued()-1) / (bitsNeeded/32)-1) * 8 + 2,
               mainy + mainheight + 10, 4, 20 );
            g.fillRect(br.x, br.y, br.width, br.height);
            nextGraphNotchAt = nextGraphNotchAt + bitsNeeded/32;
         }

         if (bitsNeeded<bytes.bitsEnqueued()) 
         {
            getMouse = false;
            //  Erase rectangle and graph.
            //g.setColor(Color.black);
	    g.setColor(backgroundColor); // May 27, 1999 fixed some problem;
 
            Rectangle ar = this.bounds();         
            g.fillRect(ar.x, ar.y, ar.width, ar.height);
            repaint();
            enterPassphrase("Now please enter a passphrase.");
         }
  
      }
      return true;
   }


   /**
    *  Handle button clicks and returns.
    */
   public boolean action(Event e, Object o)
   {
      if ((e.target==passphraseField || e.target==submitPassphraseButton)
        && passphraseField.getText().length()>0)
         confirmPassphrase();

      else if ((e.target==passphraseConfirmField || e.target==confirmPassphraseButton) 
        && passphraseConfirmField.getText().length()>0)
      {
         //  Check if user entered the same passphrase in both fields
         if (passphraseField.getText().equals(passphraseConfirmField.getText()))
            createAccount();
         else enterPassphrase("Your passphrase didn't match.  Try again."); 
      }

      else if ((e.target==usernameField || e.target==submitUsernameButton)
        && usernameField.getText().length()>0)
      {
         //  Check for validity of entered username
         username = usernameField.getText().toLowerCase();
         if (!checkUsername(username))
            enterUsername("You entered a bad username.  Try again.");
         else createAccount();
      }

      else if (e.target==loginButton)
      {
         //  Exit to login screen and destroy applet.
         try
         {
            getAppletContext().showDocument(new URL(hushHome));
         }  catch (MalformedURLException me)  { System.out.println("Malformed URL"); }
         stop();
         destroy();
      }

      return true;    
   }    


   /**
    *  This method sets up a text field for a username.
    */
   private void enterUsername(String prompt)
   {
      removeAll();
      add(this,new Label(prompt),0,0,1,1,0,0,0,0,N,NO,0,0,0,0);
      add(this, new Label(
         "Your HushMail email address will be <your-username>@hushmail.com."),
         0,1,1,1,0,0,0,0,N,NO,0,0,0,0);
      add(this,new Label(
         "Your username should be at least 6 characters but no more than 32 characters."),
         0,2,1,1,0,0,0,0,N,NO,0,0,0,0);
      add(this,new Label(
         "It should include only letters, numbers, hyphens(-), underscores(_), and dots(.)"),
         0,3,1,1,0,0,0,0,N,NO,0,0,0,0);
      format (usernameField = new TextField(32));
      add(this,usernameField,0,4,1,1,0,0,0,0,N,NO,4,0,4,0);
      format2(submitUsernameButton = new Button("Submit"));
      add(this,submitUsernameButton,0,5,1,1,0,0,0,1,N,NO,4,0,4,0);
      paintAll(getGraphics());
      usernameField.requestFocus();
   }


   private boolean checkUsername(String name)
   {
      if (name.length()< 6) return false;
      if (name.length()>32) return false;
      if (!Character.isLetter(name.charAt(0))) return false;
      for (int x=1; x<name.length(); x++)
      {
         char c=name.charAt(x);
         if (!(Character.isLetterOrDigit(c)||c=='_'||c=='-'||c=='.')) return false;
      }
      return true;
   }


   /**
    *  This method sets up a text field for a passphrase.
    */
   private void enterPassphrase(String prompt)
   {
      removeAll();
      add(this,new Label (prompt),0,0,1,1,0,0,0,0,N,NO,0,0,0,0);
      add(this,new Label ("Remember that your account is only as secure as your passphrase."),
         0,1,1,1,0,0,0,0,N,NO,0,0,0,0);
      add(this,new Label (
         "Be sure it's long and contains an unpredictable variety of characters."),
         0,2,1,1,0,0,0,0,N,NO,0,0,0,0);
      add(this,new Label (
         "If you lose or forget your passphrase your account will not be accessible."),
         0,3,1,1,0,0,0,0,N,NO,0,0,0,0);
      add(this,new Label (
         "WE CANNOT RETRIEVE YOUR PASSPHRASE.  So be sure to write it down!"),
         0,4,1,1,0,0,0,0,N,NO,0,0,0,0);
      format (passphraseField=new TextField (64));
      passphraseField.setEchoCharacter('*');
      passphraseField.setEditable(true);
      add(this,passphraseField,0,5,1,1,0,0,0,0,N,NO,4,0,4,0);
      format2(submitPassphraseButton = new Button("Submit"));
      add(this,submitPassphraseButton,0,6,1,1,0,0,0,1,N,NO,4,0,4,0);
      paintAll(getGraphics());
      passphraseField.requestFocus();
   }


   /**
    *  This method sets up a text field for passphrase confirmation.
    */
   private void confirmPassphrase()
   {
      passphraseField.setEditable(false);
      remove(passphraseField);
      remove(submitPassphraseButton);
      add(this,passphraseField,0,5,1,1,0,0,0,0,N,NO,4,0,4,0);
      add(this,new Label("Please re-enter your passphrase for confirmation"), 
         0,6,1,1,0,0,0,0,N,NO,0,0,0,0);
      format(passphraseConfirmField = new TextField(64));
      passphraseConfirmField.setEditable(true);
      passphraseConfirmField.setEchoCharacter('*');
      add(this,passphraseConfirmField,0,7,1,1,0,0,0,0,N,NO,4,0,4,0);
      format2(confirmPassphraseButton = new Button("Submit"));
      add(this,confirmPassphraseButton,0,8,1,1,0,0,0,1,N,NO,4,0,4,0);
      paintAll(getGraphics());
      passphraseConfirmField.requestFocus();
   }


   /**
    *  This method opens a socket connection and sends a message to the
    *  server to create the new account.
    */
   private void createAccount()
   {
      String passphrase = passphraseField.getText();
      
      removeAll();

      if (!keysGenerated)
      {
         // Do a SHA1 hash of all collected random bytes in blocks of twenty at a time
         byte[] privKeyBytes = bytes.dequeueBytes(KEYSTRENGTH / 8);
   
         int bytesHashed = 0;
         int remaining = privKeyBytes.length;
         while ((remaining = privKeyBytes.length-bytesHashed) > 0 )
         {
            byte[] beingHashed = new byte[ (remaining >= 20) ? 20 : remaining ];
            System.arraycopy(privKeyBytes,bytesHashed,beingHashed,0,beingHashed.length);
            byte[] hashedBlock = new HushSHA1().SHA1Hash(beingHashed);
            System.arraycopy(hashedBlock,0,privKeyBytes,bytesHashed,beingHashed.length);
            bytesHashed = bytesHashed + beingHashed.length;
         }

         ElGamalKeys keys = new ElGamalKeys(privKeyBytes);

         //  wipe array containing source of private key
         for (int x = 0; x < privKeyBytes.length; x++) privKeyBytes[x]=0;
         privKeyBytes = null;

         BlowfishCipher encPriv = new BlowfishCipher();

         encPriv.setKey(passphrase.getBytes());

         publicKey = Conversions.bytesToHexString(keys.publicKey());
     
         encPrivateKey = Conversions.bytesToHexString(encPriv.encrypt(keys.privateKey()));

         encPriv = null;

         /* Pause here while key generation is completed.
          * After key generation: 
          *   publicKey = <a hex string representing the users public key>
          *   encPrivateKey = <a hex string representing the users private key blowfish
          *     encrypted by the passphrase>
          */
         keysGenerated = true;
    
      }

      byte[] passphraseHashBytes = new HushSHA1().SHA1Hash(passphrase);

      byte[] halfPassphraseHashBytes = new byte[passphraseHashBytes.length/2];
      System.arraycopy(passphraseHashBytes, 0,
         halfPassphraseHashBytes, 0, halfPassphraseHashBytes.length);
      String passphraseHash = Conversions.bytesToHexString(halfPassphraseHashBytes);
      try
      {
         Socket s =  getSocket(); //new Socket(serverAddress,serverPort);
         s.setSoTimeout(60000);
         eIn = new BufferedInputStream(s.getInputStream());
         out = new BufferedOutputStream(s.getOutputStream());

         /* Write session ID unencrypted so the server
          * can find the session key.
          */
         out.write(sessionIDBytes);
 
         int infoCount = 4;
         for (int x=0; x<userinfo.length(); x++)
            if (userinfo.regionMatches(x,"\n",0,1)) infoCount++;
     
         //  Send encrypted user information to the server
         encryptAndWrite("NEWACCOUNT\n" + username +"\n"+ infoCount +"\n"+ publicKey +"\n"+
            passphraseHash +"\n"+ encPrivateKey +"\n"+ diskspace +"\n"+ userinfo);

         readAndDecrypt();
         eIn.close();
         out.close();
         s.close();

         if (!in.readLine().equals("OK")) error();

         String response = in.readLine();

         if (response.equals("NOT APPROVED"))
            enterUsername("The username you selected is unavailable.  " +
                          "Please enter a different username.");  
         else if (response.equals("APPROVED"))
         {

            //  Wipe Passphrase
            //  Wipe passphrase fields

            passphraseField.setText("");
            passphraseConfirmField.setText("");
            passphrase = "                                                               "+
                         "                                                               "+
                         "                                                               "+
                         "                                                               ";
            passphrase = null;

            removeAll();  
            add(this, new Label("Your account has been created!"),
               0,0,2,1,0,0,0,0,N,NO,0,0,0,0);
            add(this, new Label("Your username is:"),0,1,1,1,0,0,.5,0,N,NE,0,0,0,0);
            Label usernameLabel = new Label(username);
            usernameLabel.setForeground(color);
            add(this, usernameLabel,1,1,1,1,0,0,.5,0,N,NW,0,0,0,0);
            add(this, new Label("Your email address is:"),0,2,1,1,0,0,.5,0,N,NE,0,0,0,0);
            Label emailAddressLabel=new Label(username+"@hushmail.com");
            emailAddressLabel.setForeground(color);
            add(this, emailAddressLabel,1,2,1,1,0,0,.5,0,N,NW,0,0,0,0);
            format2(loginButton = new Button("Click here to login"));
            add(this, loginButton,0,4,2,1,0,0,0,1,N,NO,4,0,0,0);
            paintAll(getGraphics());
         }
         else error();

      }  catch (IOException ie) {error();}
         catch(IllegalMonitorStateException imse) {}
   }


   private void error()
   {
      removeAll();
      add(this, new Label(
         "I'm sorry.  There has been an error."),    0,0,1,1,0,0,0,0,N,NO,0,0,0,0);
      add(this, new Label("Please try again later."),0,1,1,1,0,0,0,0,N,NO,0,0,0,0);
      format2(loginButton = new Button("Return home"));
      add(this,loginButton,0,2,1,1,0,0,0,1,N,NO,4,0,0,0);
      paintAll(getGraphics());
   }  


   private void format(Component c)
   {
      c.setForeground(Color.black);
      c.setBackground(Color.white);
      c.setFont(new Font("Helvetica",Font.PLAIN,12));
   }

   private void format2(Component c)
   {
      c.setForeground(Color.black);
      c.setBackground(new Color(192,192,192));
      c.setFont(new Font("Helvetica",Font.PLAIN,12));
   }

   /**
    *  Variables and methods for handling GridBagLayouts
    */
   private int C = GridBagConstraints.CENTER;
   private int NW = GridBagConstraints.NORTHWEST;
   private int NE = GridBagConstraints.NORTHEAST;
   private int NO = GridBagConstraints.NORTH;
   private int B = GridBagConstraints.BOTH;
   private int H = GridBagConstraints.HORIZONTAL;
   private int N = GridBagConstraints.NONE;
   private int R = GridBagConstraints.REMAINDER;


   /**
    *  This method adds a Component to a Container which uses a GridBagLayout.
    */
   private void add(Container cc, Component comp, int x, int y, int w, int h, int ipx, int ipy,
      double wtx, double wty, int fill, int anchor, int t, int l, int b, int r) 
   {
      GridBagConstraints constraints = new GridBagConstraints();
      constraints.gridx = x;
      constraints.gridy = y;
      constraints.gridwidth = w;
      constraints.gridheight = h;
      constraints.ipadx = ipx;
      constraints.ipady = ipy;
      constraints.weightx = wtx;
      constraints.weighty = wty;
      constraints.fill = fill;
      constraints.anchor = anchor;
      constraints.insets = new Insets(t,l,b,r);
      LayoutManager ll = cc.getLayout();
      GridBagLayout g = (GridBagLayout)ll;
      g.setConstraints(comp, constraints);
      cc.add(comp);
   } 


   /**
    *  This methods send the parameter string to the
    *  server, calling appropriate encryption methods 
    *  and indicating the length.
    */
   private void encryptAndWrite(String output)
   {
      try
      {
         //  Send 16 bytes indicating the length of the output to follow
         byte[] encOutputBytes = blowfishPipe.stringEncrypt(output); 
         int outLength = encOutputBytes.length;
         out.write(blowfishPipe.encrypt(Conversions.intToBytes(outLength)));
         out.write(encOutputBytes);
         out.flush();	
      }  catch (IOException e) {System.out.println("IOException");}
   }


   /**
    *  This method reads in a set of bytes from the server
    *  calls appropriate decryption methods, and stores a resulting
    *  string in a buffer for later access.
    */
   private void readAndDecrypt() throws IOException
   {
      //  read in 16 bytes indicating the length of the input to follow
      byte lengthBytes[] = new byte[16];
      int bytesRead = eIn.read(lengthBytes);
      while (bytesRead<16)
         bytesRead = bytesRead + eIn.read(lengthBytes, bytesRead, 16-bytesRead);

      int inLength = Conversions.bytesToInt(blowfishPipe.decrypt(lengthBytes));

      //  read and decrypt the requested number of bytes
      byte[] eBuf = new byte[inLength];

      bytesRead = eIn.read(eBuf);
      while (bytesRead<inLength)
         bytesRead = bytesRead + eIn.read(eBuf, bytesRead, inLength-bytesRead);
  
      String inString = blowfishPipe.stringDecrypt(eBuf);

      byte[] trimmedBytes = new byte[inString.length()];
      inString.getBytes(0, inString.length(), trimmedBytes, 0);
      in = new DataInputStream(new ByteArrayInputStream(trimmedBytes));
   }
   
 /**  Version 1.04 The following is a section dealing 
  with using multithreads to make connections */
  
  
 // MultiConnectionThreadGroup mctg;

  //StringBuffer lock = new StringBuffer();
  //static boolean notifyControl;
  public synchronized Socket getSocket () throws IOException
  {
    serverPort = portArray[0];
    Socket tempSocket = new Socket(serverAddress, serverPort);
    /* notifyControl = true;

    if (serverPort != -1)
    {
        socket = new Socket(serverAddress, serverPort);
        return socket;
    }
    mctg = new MultiConnectionThreadGroup();

    mctg.start();

    long timer = System.currentTimeMillis();
    while ( socket == null)
    {
        if (System.currentTimeMillis() - timer > 60000)
            break; // time out
        try
        {
            lock.wait(60000);
        } catch (Exception e)
        {
           // if (bDebug)
             // System.out.println("????");
           // throw new IOException();
        }

    }
     //mctg.stop();
     if ( socket != null)
        notifyControl = false;
     else
        throw new IOException("Network Connection error");
      */
      return tempSocket;
  }

  /* class MultiConnectionThreadGroup // extends ThreadGroup
  {
    ConnectionThread ct[]= new ConnectionThread[ portArray.length];

    MultiConnectionThreadGroup ( )
    {
        //super("Connection");
        for ( int i=0; i<portArray.length; i++)
        {
            ct[i] = new ConnectionThread(portArray[i]);
            //ct[i].start();
            System.err.println(" port = "+i);
        }
    }

       void start()
       { 
         for (int i=0; i<portArray.length; i++)
         {
            ct[i].start();
         }
       }
        void stop()
       {
          for ( int i=0; i<portArray.length; i++)
          {
            ct[i].stop();// = null;
          }
       }
        
  }
    
   class ConnectionThread extends Thread
   {
        int port;
        ConnectionThread ( int i)
        {
            super(""+i);
            port = i;
        }
        public void run()
        {
            
             if ( socket == null)
             {
                try 
                {
                    socket = new Socket(serverAddress, port);
                    //notify();
                }catch(Exception ex)
                {
                    return;
                }
                finally
                { 
                 
                }
                if (notifyControl)
                try { 
                  lock.notify();  
                } catch (IllegalMonitorStateException ims) {}
           }
        }
   } */




    

}  //  end NewAccountApplet

















