


/*
 *   File:  PagingConnection.java
 *
 *   Version 1.04 v1a
 *   Copyright 1998, 1999 by Hush Communications Corporation, BWI
 *
 *  Created June 4, 1999
 *  $Author Zhou Wu$
 */

package hush.net;

import java.io.*;
import java.net.*;
import java.util.Date;

import hush.util.Conversions;
//import hush.util.HushRandom;
import hush.security.HushSHA1;
import hush.security.BlowfishCipher;

/** Overwrite some methods in Conncetion class for paging use
 */

public class PagingConnection //extends Connection
{

    
    /* An array of public key fingerprints
     */
    private String[] fingerprintsArray;
    public String fingerprints(int x) {return fingerprintsArray[x];}


/**
 * The following variables and methods deal with information  
 * about the users account and current session.
 */

  /* The server's address
   */
  private String serverAddress;

  /* The port the server listens for connections on
   */
  private int serverPort = 80;

  /* The socket used to communicate with the server
   */
  protected Socket s;  
 
  /* The service's domain.  Usually hushmail.com
   */
  private String domain;
  public String domain() {return domain;}

  /* The username
   */
  private String username; 
    public String username() {return username;}

  /* The users email address
   */
  private String emailAddress;
  public String emailAddress() {return emailAddress;}  


  /*  The user's public key fingerprint
   */
  private String fingerprint = "";
  public String fingerprint() {return fingerprint;}

  /* The cipher with the key as user's passphrase
   */
  private BlowfishCipher passCipher;
  public void killPassCipher() {passCipher=null;}

  /* The pseudo stream cipher for communication between applet and server
   */
  private BlowfishCipher blowfishPipe;
 
  /* The sessionID.  64 bytes used as a unique identifier by the server to ensure
   * a user cannot open multiple sessions, and that no one can break into the middle
   * of a session.
   */
  private byte[] sessionIDBytes;

  /* The user's encrypted public key
   */
  private byte[] encPrivKey;


  /* A boolean flag that indicates if the socket connection has been used yet.
   * This ensures the session will only be initialized by method use() once
   */
  private boolean initialUse;


 /* An output stream to the socket
   */
  BufferedOutputStream out;

  /* An input stream from the socket.
   */
  BufferedInputStream eIn;

  /* An input stream from a decrypted String which is used as a buffer
   * to store data from eIn after it has been decrypted with blowfishPipe
   */
    //DataInputStream in;
    BufferedReader in;

    //BufferedReader reader;

  /* A boolean flag that indicates if the socket connection has been used yet.
   * This ensures the session will only be initialized by method use() once
   */

  /* A count of the tasks currently requiring use of the socket.
   * Allows the socket to close when not needed.
   */
  protected int socketRequests;



  /* Half of a hash of the passPhrase;
   */
  private byte[] halfPassphraseHash;

  public void setPassphrase(String passedPassphrase) 
  {
    byte[] passphraseHash = new HushSHA1().SHA1Hash(passedPassphrase);
    halfPassphraseHash=new byte[passphraseHash.length/2];
    System.arraycopy(passphraseHash,0,halfPassphraseHash,0,halfPassphraseHash.length);
    passphraseHash = null;
    passCipher = new BlowfishCipher();
    passCipher.setKey(passedPassphrase.getBytes());
    passedPassphrase = "                                            " +
                       "                                            " +
                       "                                            " +
                       "                                            " +
                       "                                            " ;
    passedPassphrase=null;
  }
 
  
    /** Constructor 
     * @param passUsername a String of user name
     * @param passedSessionID a String of Session ID
     * @param SessionKey  a String of Session Key
     */


   public PagingConnection(String passedUsername, String passedSessionID, String passedSessionKey)
	{
	    //super(passedUserName, passedSessionID, passedSessionKey);
	    
	    /** 
	     * This is the class constructor
	     */
	    username = passedUsername.toLowerCase();
	    
	    /* Convert sessionID hex string to an array of bytes
	     */
	    sessionIDBytes = Conversions.hexStringToBytes(passedSessionID);  

	    /* Convert sessionKey hex string to an array of bytes
	     */
	    byte[] sessionKeyBytes = Conversions.hexStringToBytes(passedSessionKey);

	    /* Instantiate cipher with sessionKey
	     */
	    blowfishPipe = new BlowfishCipher();
	    blowfishPipe.setKey(sessionKeyBytes);

	    //hushApplet = a;
   
	    //serverAddress = hushApplet.getCodeBase().getHost();

	    //if(serverAddress.trim().equals("")) // This is a local host! Zhou Wu
	     serverAddress = Connection.TEST_SERVER_ADDRESS; 

	    domain = "hushmail.com";
	    emailAddress = username+"@"+domain;

	    /* Since this session has just been started, no tasks are using the socket yet
	     */
	    socketRequests = 0;

	    /* The socket has not yet been used for communication
	     */
	    initialUse = true;

	    //randStream = new HushRandom(new HushSHA1().SHA1Hash(Conversions.longToBytes(new Date().getTime())));


        }



    

    public void setServerAddress(String ipAddress)
	{
	    serverAddress = ipAddress;
	}

  /**
   *  This method should be called when a task requires 
   *  the use of the socket connection 
   */
    
   

    public boolean use() throws IOException, NoRouteToHostException
	{
	    //hushApplet.startBlinkingText();

	    if (initialUse)
	    {
		/* Increment count of threads using socket
		 */
		++socketRequests;
		
		s = new Socket(serverAddress,serverPort);

		s.setSoTimeout(60000);
		s.setTcpNoDelay(false); 

		out = new BufferedOutputStream(s.getOutputStream());
		eIn = new BufferedInputStream(s.getInputStream());

		/* Send sessionID to server, so that the server can find the sessionKey
		 */
		out.write(sessionIDBytes);

		/* Send username and half of hash of passphrase
		 */
		encryptAndWrite("CONNECT\n"+username+"\n"+Conversions.bytesToHexString(halfPassphraseHash)+"\n");

		/* Check for acceptance of username and passphrase
		 */
		
		readAndDecrypt();
			
		if (!in.readLine().equals("OK")) throw new IOException();        
		if (!in.readLine().equals("APPROVED"))
		{
		    System.err.println("Not Approved");
		    /*  Exit to "BadLogin" page
		     */
		    //System.err.println("bad login");
		    /* try 
		       {
		       URL u = new URL(hushApplet.exitpage+"?action=BadLogin&username="+username());
		       hushApplet.getAppletContext().showDocument(u);
		       }  
		       catch (MalformedURLException m) {
		       m.printStackTrace();
		       System.out.println("MalformedURL");
		       }*/    
		    eIn.close();
		    out.close();
		    s.close();
		    return false;
		}

		String publicKey = in.readLine();
		if (publicKey!=null) 
		    fingerprint = Conversions.bytesToHexString(new HushSHA1().SHA1Hash(Conversions.hexStringToBytes(publicKey)));
		String encPrivKeyString = in.readLine();      
		if (encPrivKeyString!=null)
		    encPrivKey = Conversions.hexStringToBytes(encPrivKeyString);
		initialUse = false ;
	    }
	    else
	    {
		/* Execute only if no other threads are currently using the socket
		 */
		if (++socketRequests==1)
		{
		    s = new Socket(serverAddress,serverPort);
		    s.setSoTimeout(180000);
		    s.setTcpNoDelay(true);
		    out = new BufferedOutputStream(s.getOutputStream());
		    eIn = new BufferedInputStream(s.getInputStream());

		    /* Send sessionID to server, so that the server can find the sessionKey
		     */
		    out.write(sessionIDBytes);
		    
		    encryptAndWrite("RECONNECT\n"+username+"\n");

		    /* Check for approval for continuation of session
		     */
		    readAndDecrypt();
		    if (!in.readLine().equals("OK")) throw new IOException();       
		    if (!in.readLine().equals("APPROVED"))
		    {
			System.out.println("Reach note Approved");
			//try
			//{
			    //URL u = new URL(hushApplet.exitpage+"?action=NotApproved&username="+username());
			    //hushApplet.getAppletContext().showDocument(u);
			    //hushApplet.removeAll();
			    //hushApplet.paintAll(hushApplet.getGraphics());
			    //wipeMessage();
			    killPassCipher();
			    //}  
			    //catch (MalformedURLException m)
			    //{
			    //m.printStackTrace();
			    //System.out.println("MalformedURL");
			    ///} 
			    eIn.close();
			    out.close();
			    s.close();
			    return false;
		    }
		}
	    }
	    return true;
	}


    /**
     *  This method should be called when a task using
     *  the socket connection has been completed
     */
    public  void done()
	{
	    //hushApplet.stopBlinkingText();

	    if (s==null) return;

	    /* decrement count of threads using socket
	     * if count is zero then terminate the socket
	     */
	    if (--socketRequests==0)
	    {
		try {
		    out.close();
		    eIn.close();
		    s.close();
		} catch (IOException e1) {System.out.println("IOException");}
	    }
	}


    /*  This signals the server to delete the session record
     *  and terminates the socket
     */
    public  void quit()
	{
	    if (s==null) return;
	    try
	    {
		encryptAndWrite("QUIT\n");
		out.close();
		eIn.close();
		s.close();
	    } catch (IOException e1) {
		System.out.println("IOException");
	    }
	}




  /**
   *  This method reads in a set of bytes from the server
   *  calls appropriate decryption methods, and stores a resulting
   *  string in a buffer for later access.
   */
  private  void readAndDecrypt() throws IOException
  {
    /* read in 16 bytes indicating the length of the input to follow 
     */
    byte lengthBytes[] = new byte[16];

    int bytesRead = eIn.read(lengthBytes);
    while (bytesRead<16) bytesRead=bytesRead+eIn.read(lengthBytes,bytesRead,16-bytesRead);
      
    int inLength = Conversions.bytesToInt(blowfishPipe.decrypt(lengthBytes));

    if (inLength==0) throw new IOException();

    /* read and decrypt the requested number of bytes
     */
    byte[] eBuf = new byte[inLength];

    bytesRead = eIn.read(eBuf);
    while (bytesRead<inLength) bytesRead=bytesRead+eIn.read(eBuf,bytesRead,inLength-bytesRead);  

    String inString = blowfishPipe.stringDecrypt(eBuf);

    //byte[] trimmedBytes = new byte[inString.length()];
    //inString.getBytes(0,inString.length(),trimmedBytes,0);
    byte[] trimmedBytes = inString.getBytes();
  
    in = new BufferedReader(new InputStreamReader(new ByteArrayInputStream(trimmedBytes)));
   
  }


  /**
   *  This method reads in a set of bytes from the server
   *  calls appropriate decryption methods, and stores a resulting
   *  string in a buffer for later access.
   */
  public  String readAndDecryptWithResult() throws IOException
  {
    /* read in 16 bytes indicating the length of the input to follow 
     */
    byte lengthBytes[] = new byte[16];

    int bytesRead = eIn.read(lengthBytes);
    while (bytesRead<16) bytesRead=bytesRead+eIn.read(lengthBytes,bytesRead,16-bytesRead);
      
    int inLength = Conversions.bytesToInt(blowfishPipe.decrypt(lengthBytes));

    if (inLength==0) throw new IOException();

    /* read and decrypt the requested number of bytes
     */
    byte[] eBuf = new byte[inLength];

    bytesRead = eIn.read(eBuf);
    while (bytesRead<inLength) bytesRead=bytesRead+eIn.read(eBuf,bytesRead,inLength-bytesRead);  

    String inString = blowfishPipe.stringDecrypt(eBuf);

    //byte[] trimmedBytes = new byte[inString.length()];
    //inString.getBytes(0,inString.length(),trimmedBytes,0);
    byte[] trimmedBytes = inString.getBytes();
    in = new BufferedReader(new InputStreamReader(new ByteArrayInputStream(trimmedBytes)));
    
  // TODO: more work to be done here!!! 
    return in.readLine();
  }


  /**
   *  This methods send the parameter string to the
   *  server, calling appropriate encryption methods 
   *  and indicating the length.
   */
  public void encryptAndWrite(String output)
  {
    try
    {
    /* Send 16 bytes indicating the length of the output to follow
     */
      byte[] encOutputBytes = blowfishPipe.stringEncrypt(output); 
      int outLength = encOutputBytes.length;
      out.write(blowfishPipe.encrypt(Conversions.intToBytes(outLength)));
      out.write(encOutputBytes);
      out.flush();	
    } catch (IOException e) {System.out.println("IOException");}
  }

} 
