/*
 *   File:  ByteQueue.java
 *
 *   This Class queues up bits, then returns them later as needed.
 *   Used for storing truly random entropy from keystrokes of the user.
 *
 *   version 1.04 v1a
 *   Copyright 1998, 1999 by Hush Communications Corporation, BWI
 */

package hush.util;

import java.util.Vector;
//import hushcode.Conversions;


public final class ByteQueue
{
   private Vector bytes;
   private int currentByte;
   private int currentBit;
   private int bitsEnqueued;
   private Byte zero = new Byte((byte)0);


   public ByteQueue()
   {
      bytes = new Vector();
      currentByte = 0; 
      currentBit = 7;
      bytes.addElement(zero);
   }


   /*  This method accepts a number and enqueues the numBits least significant bits.
    *  The number must be positive, and no more than 63 bits may be enqueued at a time.
    */
public    void enqueueBits(long number, int numBits)
   {   
      long p2 = (long)(Math.pow(2,(double)(numBits-1)));
      if (number-p2 > p2) number = number%(p2*2);
      for (long n=p2; ;n=n/2)
      {
         if (number%n==number) currentBit--;
         else 
         {
            byte newByte = (byte)
               ((int)((Byte)(bytes.elementAt(currentByte))).byteValue() | (1<<currentBit--));
            bytes.setElementAt(new Byte(newByte),currentByte);
         }
         if (currentBit<0)
         {
            currentBit = 7;
            currentByte++;
            bytes.setSize(currentByte);
            bytes.addElement(zero);
         }
         number = number % n;
         if (n==1) break;
      }
      bitsEnqueued = bitsEnqueued+numBits;    
   }


   public int bitsEnqueued() { return bitsEnqueued; }

   public int bytesAvailable() { return currentByte; }

   public int longsAvailable() { return currentByte/8; }


public   byte[] dequeueBytes(int numBytes)
   {
      byte[] returnBytes = new byte[numBytes];
      for (int n=0; n<numBytes; n++)
         returnBytes[n] = dequeueByte();
      return returnBytes; 
   }


 public  byte dequeueByte()
   {
      byte returnByte = ((Byte)bytes.firstElement()).byteValue();
      bytes.removeElementAt(0);
      currentByte--;
      return returnByte;
   }


 public  long dequeueLong()
   {
      byte[] rBytes = new byte[8];
      for (int n=0; n<8; n++)
         rBytes[n] = dequeueByte();
      return Conversions.bytesToLong(rBytes); 
   }

}   //  end ByteQueue
